package gggt.eg.bui;

import gom.*;
import gom.std.*;

import gggt.eg.set.*;
import gggt.eg.dst.*;

/**
* The building class is the support of build class.
*/
public abstract class Building {


	/** Default constructor. */
	Building()
		{init();}


	/** To init from a given set. */
	public void init(Setting set) {
		init();
		if(set!= null) {
			this.set = set;
			alg = set.getAlgorithm();
			Noise n = set.getNoise();
			if(n.getApply())
				dist = new Distort(n.getResize(),n.getRotation(),n.getMove(),n.getVariance());
			map = set.getMap();
			}
		}
	private void init()
		{sinit(); binit(); rinit();}
	private void sinit() // to init the setting
		{set = null; alg = null; dist = null; map = null;}
	private Setting set; private Algorithm alg; private Distort dist; private gggt.eg.set.Map map;
	
	private void binit() // to init the building parameter
		{ doc = null; fuse = osize = 0; }
	private OHL doc;
	private int osize, fuse;
	private void rinit() // to init the result parameter
		{rdoc = null; q = 0;}
	private OHL rdoc; private double q;


	/** To get the building attribute. */
	public String getBuild() {
		if(alg != null)
			return alg.getBuild();
		return null;
		}


	/** To start the build process. */
	public void start() {
		binit();
		if(set!=null)
			{doc = new OHL(); set.start(); }
		}

	/** To start a building loop. */
	public void loop()
		{ c = null; m = null; s = null; }
	private Constraint c; private Model m; private OSymbol s;

	/** The factory method selects a model and build a corresponding symbol. */
	public void factory() {
		if(set != null) {
			if(set.hasRigid()) {
				c = set.nextRigid();
				if(c!=null) 
					m = c.nextModel();
				s = generate(m);
				}
			else {
				m = set.nextModel();
				if(m!=null)
					c = m.nextConstraint();
				s = generate(m);
				}
			}
		}
	private OSymbol generate(Model m) {
		if((map != null)&(m != null)) {
			OSymbol s = new OSymbol(m.getGraphic(),m.getName());

			s.scale(m.getScale());
			s.resize(m.getResize());
			s.morph(m.getMorph());
			s.wrap(m.getWrap());

			return s;
			}
		return null;
		}

	/** To perform distort operation on given symbol. */
	public void distort() {
		if((dist!=null)&(s!=null))
			dist.process(s.getObject());
		}

	/** To constrain a symbol. */
	public void constrain() {
		if((c!=null)&(s!=null))
			c.constrain(s);
		}

	/** Test if the given symbol satisfy the constraint. */
	public boolean satisfy() {
		if(c!=null) {
			if(c.full())
				return false;
			if( (c.getOverflow()==false) & overflows() )
				return false;
			if( (c.getOverlap()==false) & overlaps() )
				return false;
			if( (c.getOutside()==false) & outside() )
				return false;
			return true;
			}
		return false;
		}

	private boolean overflows() {
		if((c!=null)&(s!=null))
			return c.overflows(s);
		return false;
		}
	private boolean overlaps() {
		if((doc!=null)&(s!=null)) {
			ORectangle r1 = new ORectangle(s.boundingBox()), r2;
			for(int i=0;i<doc.objSize();i++)
				if(s!=doc.getObj(i)) {
					r2 = new ORectangle(doc.getObj(i).boundingBox());
					if(r1.overlaps(r2))
						return true;
					}
			}
		return false;
		}
	private boolean outside() {
		if((map!=null)&(s!=null))
			return !map.includes(new ORectangle(s.boundingBox()));
		return true;
		}

	public void mark() { 
		System.out.println("---------------------------------------------------------------------------");
		System.out.println("---------------------------------------------------------------------------");
		System.out.println("---------------------------------------------------------------------------");
		System.out.println("---------------------------------------------------------------------------");
		}

	public void debug() { 
		if((c!=null)&(s!=null)) {
			System.out.println("---------------------------------------------------------------------------");
			System.out.println("fuse:"+fuse+" osize:"+osize);
			System.out.println("---------------------------------------------------------------------------");
			c.print(); s.print();
			System.out.println("overflow:\t"+ overflows());
			System.out.println("overlap:\t"+ overlaps());
			System.out.println("outside:\t"+ outside());
			System.out.println("satisfy:\t"+ satisfy());
			}
		}

	/** To add the given symbol to document. */
	public void add() {
		if((c!=null)&(doc!=null)&(s!=null)) {
			if(c.getOverlap()==true)
				{doc.addAtt(s); c.grow();}
			else
				{doc.addObj(s); c.grow();}
			}
		}

	/** To put the given symbol in document. */
	public void put()
		{ destroy(); add(); }
	private void destroy() {
		if((doc!=null)&(s!=null)) {
			ORectangle r1 = new ORectangle(s.boundingBox()), r2;
			for(int i=0;i<doc.objSize(); i++) {
				r2 = new ORectangle(doc.getObj(i).boundingBox());
				if(r1.overlaps(r2))
					{doc.removeObj(i); i--;}
				}
			}
		}

	/**
	* To run the build process.
	* This method increases the cycle member and return false
	* when object number is equal to wished object number
	* or when current cycle is upper than set cycle.
	*/
	public boolean run() {
		if((doc != null)&(map != null)) {
			if(getSize() <= osize)
				fuse++;
			osize = getSize();
			if	(
				(getSize() < map.getSize())
				&
				(getFuse() >= fuse)
				)
				return true;
			}
		return false;
		}
	private int getSize()
		{return doc.attSize()+doc.objSize();}

	private int getFuse()
		{return (int)Math.floor(alg.getFuse()*(double)map.getSize());}

	/** 
	* To scale the final document. 
	*/
	public void scale() {
		if((doc != null)&(map != null))
			doc.scale(map.getScale(),z);
		}
	private double[] z = new double[2];
	
	/** To stop the build process. */
	public void stop() {
		if(doc != null) {
			rinit();
			rdoc = doc;
			q = ((double)(doc.objSize()+doc.attSize()))/((double)map.getSize())*100;
			binit();
			}
		}

	/** Return the built document. */
	public OHL getDoc()
		{return rdoc;}

	/** Return the building quality factor. */
	public double getQ()
		{return q;}

	/*
	private void debug() {
		if(doc != null) {
			int i=0; String name=null;
			do
				{name = "build_"+String.valueOf(i)+".svg"; i++;}
			while(new File(name).exists());
			Write.svgWrite(doc,map.getWidth(),map.getHeight(),name);
			}
		}
	*/
	/*
	public void insert() {
		if((doc!=null)&(s!=null)) {
			ORectangle r1 = new ORectangle(s.boundingBox()), r2; double[] delta;
			for(int i=0;i<doc.objSize(); i++) {
				r2 = new ORectangle(doc.getObj(i).boundingBox());
				if(r2.overlaps(r1))
					{delta = r2.eject(r1); s.move(delta[0],delta[1]);}
				}
			if(includes())
				if(!overlaps())
					add();
			}
		}
	/*
	public void push() {
		if((doc!=null)&(s!=null)) {
			ORectangle r1 = new ORectangle(s.boundingBox()), r2;
			OHL delete = new OHL();
			for(int i=0;i<doc.objSize(); i++) {
				r2 = new ORectangle(doc.getObj(i).boundingBox());
				if(r1.overlaps(r2)) {
					delete.addObj(doc.getObj(i));
					doc.removeObj(i);
					i--;
					}
				}
			add();
			for(int i=0;i<delete.objSize();i++)
				insert((OSymbol)delete.getObj(i));
			}
		}
	*/

	}

