package gggt.eg.dst;

import java.util.*;

/**
* The Gaussian class.
*/
public class Gaussian extends Random {

	private static final long serialVersionUID = 1;

	private static final int smax = 20;
	private static double[] c0;
	private static int[] c1;
	private static double[] c2;

	private double vmax;
	private double pmax;
	private double var;

	/** return the vmax value */
	private double getVMax()
		{return vmax;}

	/** return the pmax value */
	private double getPMax()
		{return pmax;}

	/** To return the var value. */
	public double getVar()
		{return var;}

	/** Constructor with a var parametter. */
	public Gaussian(double var) {
		c0 = new double[smax];
		for(int i=0;i<smax;i++)
			c0[i] = Math.pow(-1,i);

		c1 = new int[smax];
		for(int i=0;i<smax;i++)
			c1[i] = 2*i+1;

		c2 = new double[smax];
		for(int i=0;i<smax;i++) {
			c2[i] = fact(i)*(2*i+1);
			}

		if(var > 0)
			this.var = var;
		else
			var = 1;
		vmax = searchMax();
		pmax = nerf(vmax)*2;
		}

	/** Constructor with var equals to 1. */
	public Gaussian() {
		this(1);
		}

	/** factoriel */
	private double fact(int n) {
		double r = 1;
		if((n>0) && (n<smax)) {
			r = (double)n;
			for(int i=n-1;i>=1;i--)
				r = r*i;
			}
		return r;
		}

	/** nerf(z) method */
	private double erf(double z) {
		double r=0;
		for(int i=0;i<smax;i++)
			r += c0[i] * Math.pow(z,c1[i]) / c2[i];
		r = r*2/Math.sqrt(Math.PI);
		return r;
		}

	/** return the normalized nerf  */
	private double nerf(double z) {
		double r = z;
		r = r/Math.sqrt(2);
		r = r/Math.sqrt(var);
		r = erf(r);
		r = r/2;
		return r;
		}

	/** search the maximum */
	private double searchMax() {
		double z=0;
		double p1=0,p2=0,delta=0;
		while(delta>=0) {
			p2 = nerf(z);
			delta=p2-p1;
			p1=p2;
			z += gap;
			}
		return z;
		}
	private static final double gap=0.001;

	/**
	* Return a z value according to
	* a given probability.
	* This method is based on the
	* use of the nerf method.
	* It uses a dichotomic approach.
	*/
	private double getZ(double p) {

		double v1=0,p1;
		double v3=vmax,p3;
		double v2 = v1 + (v3-v1)/2,p2;
		double delta;
		for(int i=0;i<itt;i++) {
			p1 = nerf(v1);
			p2 = nerf(v2);
			p3 = nerf(v3);

			if(p == p1)
				return v1;
			else if(p==p3)
				return v3;

			delta = p2-p;
			if(delta<0) {
				v1 = v2;
				v2 = v1 + (v3-v1)/2;
				}
			else if(delta == 0)
				return v2;
			else if(delta > 0) {
				v3 = v2;
				v2 = v1 + (v3-v1)/2;
				}
			}

		return v2;
		}
	private static final int itt=20;

	/**
	* To return a double according
	* a gaussian probability.
	*/
	public double nextValue() {
		double p = nextDouble();
		p = Math.min(p,pmax);

		if(p < 0.5)
			return -getZ(p);
		else if (p == 0.5)
			return 0;
		else
			return getZ(p-0.5);
		}


	}

