package gggt.eg.set;

import java.util.*;

import gom.std.*;

/**
* This class corresponds to a constraint tag
* into the setting file.
*/
public abstract class Constraint extends StringTo {

	private String name;			// the label of constraint

	private int stack,size; 		// the object size

	private Values scale; 			// the scale gap
	private double[] wrap; 			// the wrap values
	private Values resize; 			// the resize value
	private Values morph; 			// the morph value

	private boolean rigid;			// is it a rigid constraint
	private boolean overflow;		// is it an overflow constraint
	private boolean overlap;		// is it an overlap constraint
	private boolean outside;		// is it an outside constraint

	private Values roll;			// the roll parameter
	private double[] polar;			// the polar parameter
	private Values slope;			// the slope parameter

	private Vector<Model> models;	// the links to models

	/** Default constructor. */
	Constraint() {
		name = "none";
		stack = 0; size = 1;
		scale = new Values(1); wrap = new double[2];
		resize = new Values(1); morph = new Values(1); 
		rigid = overflow = true; overlap = outside = false;
		roll = new Values(0); polar = new double[2]; slope = new Values(0);
		models = new Vector<Model>();
		}

	/** Utility. */

	private static Random r = new Random();

	double nextDouble()
		{ return r.nextDouble(); }
	
	double nextDouble(double ssize,double csize) {
		if((ssize>0)&(csize>0)&(ssize < csize))
			return (ssize/2 + nextDouble()*(csize-ssize))/csize;
		return nextDouble();
		}

	int nextInt(int max)
		{ return r.nextInt(max); }
	
	boolean check(double[] ad,int length) {
		if((ad != null))
			if(ad.length == length)
				return true;
		return false;
		}

	boolean unit(double v) {
		if((v>=0)&(v<=1))
			return true;
		return false;
		}
	

	/** To get the constraint name. */
	public String getName()
		{return new String(name);}
	void setName(String name) {
		if(name!= null)
			this.name = new String(name);
		}

	/** To get the constraint type. */
	public abstract String getCName();

	/** To get the sizing parameter. */
	public int getSize()
		{return size;}
	void setSize(String v) {
		if(v != null)
			if((si = getInteger(v)) > 1)
				size = si;
		}
	private int si;
	/** To grow the constraint stack */
	public void grow()
		{stack ++; }
	/** Test is the stack is full. */
	public boolean full()
		{if(stack<size) return false; return true; }
	/** To clear the stack. */
	public void down()
		{stack = 0; }

	/** To get the scale parameter. */
	public double getScale()
		{ return scale.get(); }
	void setScale(String v)
		{ scale.set(v); }

	/** To get the wrap parameters. */
	public double[] getWrap()
		{  wc = new double[2]; wc[0] = wrap[0]; wc[1] = wrap[1]; return wc; }
	private double[] wc;
	void setWrap(String v) {
		if(v != null) {
			String[] tab = v.split(";");
			if(tab.length==2) {
				wrap[0] = getDouble(tab[0]);
				wrap[1] = getDouble(tab[1]);
				}
			}
		}

	/** To get the resize parameter. */
	public double getResize()
		{ return resize.get(); }
	void setResize(String v)
		{ resize.set(v); }

	/** To get the morph parameter. */
	public double getMorph()
		{ return morph.get(); }
	void setMorph(String v)
		{ morph.set(v); }

	/** To get the rigid parameter. */
	public boolean getRigid()
		{return rigid;}
	void setRigid(String v) {
		if(v != null)
			rigid = getBoolean(v);
		}

	/** To get the overflow parameter. */
	public boolean getOverflow()
		{return overflow;}
	void setOverflow(String v) {
		if(v != null)
			overflow = getBoolean(v);
		}

	/** To get the overlap parameter. */
	public boolean getOverlap()
		{return overlap;}
	void setOverlap(String v) {
		if(v != null)
			overlap = getBoolean(v);
		}

	/** To get the outside parameter. */
	public boolean getOutside()
		{return outside;}
	void setOutside(String v) {
		if(v != null)
			outside = getBoolean(v);
		}

	/** To get the rool parameters. */
	public double getRoll()
		{ return roll.get(); }
	void setRoll(String v)
		{ roll.set(v); }

	/** To get the polar parameters. */
	public double[] getPolar()
		{  pc = new double[2]; pc[0] = polar[0]; pc[1] = polar[1]; return pc; }
	private double[] pc;
	void setPolar(String v) {
		if(v != null) {
			String[] tab = v.split(";");
			if(tab.length==2) {
				double v1 = getDouble(tab[0]);
				double v2 = getDouble(tab[1]);
				if(unit(v1)&unit(v2))
					{polar[0] = v1; polar[1] = v2;}
				}
			}
		}

	/** To get the slope parameters. */
	public double getSlope()
		{ return slope.get(); }
	void setSlope(String v)
		{ slope.set(v); }

	/** To get the value parameters. */
	public abstract String getShape();
	abstract void setShape(String v);

	/** To get the next model attribute. */
	public Model nextModel() {
		if(models.size()>0) {
			int n = nextInt(models.size());
			System.out.println(n+" "+models.size());
			return models.get(n);
			}
		return null;
		}

	/** To get a string representation of models. */
	public String getModels() {
		StringBuffer sb = new StringBuffer();
		for(int i=0;i<models.size();i++) {
			sb.append(models.get(i).getName());
			if(i!=models.size()-1)
				sb.append(";");
			}
		return sb.toString();
		}
	void addModel(Model m) {
		if(m != null)
			models.add(m);
		}
	int modelNumber()
		{ return models.size(); }
	Model getModel(int p) {
		if((p >= 0)&(p < models.size()))
			return models.get(p);
		return null;
		}

	/** To constraint a symbol. */
	public abstract void constrain(OSymbol s);
	void handle(OSymbol s) {
		if(s != null) {
			s.rotate(getRoll()/360*2*Math.PI);

			s.scale(getScale());
			s.resize(getResize());
			s.morph(getMorph());
			s.wrap(getWrap());

			s.mark(getPolar());
			s.rotate(getSlope());
			}
		}

	/** To check a symbol. */
	public abstract boolean overflows(OSymbol s);

	/** The print method. */
	public void print() {
		System.out.println	(
							"<"
							+getCName()
							+" size=\""+size+"\""
							+" scale=\""+scale.toString()+"\""
							+" wrap=\""+wrap[0]+";"+wrap[1]+"\""
							+" resize=\""+resize.toString()+"\""
							+" morph=\""+morph.toString()+"\""
							+" rigid=\""+rigid+"\""
							+" overflow=\""+overflow+"\""
							+" overlap=\""+overlap+"\""
							+" outside=\""+outside+"\""
							+" roll=\""+roll.toString()+"\""
							+" polar=\""+polar[0]+";"+polar[1]+"\""
							+" slope=\""+slope.toString()+"\""
							+" shape=\""+getShape()+"\""
							+"/>\n"
							);
		}

	}

/**
* The Fixed constraint
*/
class Fixed extends Constraint {

	private OPoint pt;					// the point data

	Fixed()
		{ super(); pt = null; }

	public String getCName()
		{ return "fixed"; }

	void setShape(String v) {
		if(v!=null) {
			String[] tab = v.split(";");
			if(tab!=null)
				if(tab.length==2)
					pt = new OPoint(getDouble(tab[0]),getDouble(tab[1]));
			}
		}

	public String getShape() {
		if(pt != null)
			return String.valueOf(pt.getX())+";"+String.valueOf(pt.getY());
		return "0;0";
		}

	public void constrain(OSymbol s) {
		if((s != null)&(pt != null))
			{ handle(s); if(pt != null) s.anchor(pt); }
		}

	public boolean overflows(OSymbol s)
		{ return true; }


	}

/**
* The cable constraint
*/
class Sliding extends Constraint {

	private OSLine sl;	// the line data

	Sliding()
		{ super(); sl = null; }

	public String getCName()
		{ return "sliding"; }

	public void setShape(String v) {
		if(v!=null) {
			String[] tab = v.split(";");
			if(tab.length==4)
				sl = new OSLine(getDouble(tab[0]),getDouble(tab[1]),getDouble(tab[2]),getDouble(tab[3]));
			}
		}

	public String getShape() {
		if(sl != null) {
			b = sl.getBegin();
			e = sl.getEnd();
			return 	String.valueOf(b.getX())
					+";"+String.valueOf(b.getY())
					+";"+String.valueOf(e.getX())
					+";"+String.valueOf(e.getY())
					;
			}
		return "-1;0;1;0";
		}
	private OPoint b,e;

	private double[] bb;

	public void constrain(OSymbol s) {
		if((s != null)&(sl != null)) { 
			if(getOverflow())
				constrain(s,nextDouble()); 
			else {
				bb = s.getObject().boundingBox();
				ssize = Math.sqrt(Math.pow(bb[2],2)+Math.pow(bb[3],2));
				csize = sl.getLength();
				constrain(s,nextDouble(ssize,csize));
				}
			}
		}
	private double ssize,csize;
	
	private void constrain(OSymbol s,double r) {
		if((s != null)&(sl != null)&unit(r)) { 
			handle(s); pt = sl.getBetween(r); 
			if(pt != null) s.anchor(pt); 
			}
		}
	private OPoint pt;
	
	public boolean overflows(OSymbol s) {
		if((s != null)&(sl != null)) {
			bb = s.getObject().boundingBox();
			ss = new ORectangle(bb).getLines();
			for(int i=0;i<ss.length;i++)
				if(sl.covers(ss[i])==false)
					return true;
			}
		return false;
		}
	private OSLine[] ss;

	}

/**
* The zone constraint
*/
class Zone extends Constraint {

	private OPolygon po; // polygon data

	Zone()
		{ super(); po = null; }

	public String getCName()
		{ return "zone"; }

	public void setShape(String v) {
		if(v!=null) {
			String[] tab = v.split(";");
			if((tab.length%2) == 0) {
				double[][] o = new double[tab.length/2][2];
				for(int i=0;i<tab.length/2;i++) {
					o[i][0] = getDouble(tab[i*2]);
					o[i][1] = getDouble(tab[(i*2)+1]);
					}
				po = new OPolygon(o);
				}
			}
		}

	public String getShape() {
		if(po != null) {
			sb = new StringBuffer();
			for(int i=0;i < po.getSize(); i++) {
				pt = po.getPoint(i);
				sb.append(pt.getX()+";"+pt.getY());
				if(i!=(po.getSize()-1))
					sb.append(";");
				}
			return sb.toString();
			}
		return null;
		}
	private StringBuffer sb;
	private OPoint pt;

	public void constrain(OSymbol s) {
		if((s != null)&(po != null)) { 
			if(getOverflow())
				constrain(s,nextPoint()); 
			else {
				bb = s.getObject().boundingBox();
				ssize = Math.sqrt(Math.pow(bb[2],2)+Math.pow(bb[3],2));
				bb = po.boundingBox();
				csize = Math.min(bb[2],bb[3]);
				constrain(s,nextPoint(ssize,csize));
				}
			}
		}
	private double[] bb; 
	private double ssize,csize;
	
	private OPoint nextPoint() {
		if(po != null) {
			next = null; loop = 0;
			do {
				next = po.getAlong(nextDouble());
				if(loop++ > 25)
					return null;
				}
			while(!po.includes(next));
			return next;
			}
		return null;
		}
	private OPoint nextPoint(double ssize,double csize) {
		if(po != null) {
			next = null; loop = 0;
			do {
				next = po.getAlong(nextDouble(ssize,csize));
				if(loop++ > 25)
					return null;
				}
			while(!po.includes(next));
			return next;
			}
		return null;
		}
	private OPoint next; private int loop;

	private void constrain(OSymbol s,OPoint pt) {
		if((s != null)&(pt != null)) 
			{ handle(s); s.anchor(pt); }
		}
	
	public boolean overflows(OSymbol s) {
		if((s != null)&(po != null)) {
			ps = new ORectangle(s.getObject().boundingBox()).getPoints();
			for(int i=0; i<ps.length;i++)
				if(!po.includes(ps[i]))
					return true;
			}
		return false;
		}
	private OPoint[] ps;

	}
