package gggt.eg.set;

import java.io.*;
import java.util.*;

import gom.*;
import gom.lrw.*;

/**
* The Model class corresponds to a model tag
* into the setting file.
*/
public class Model extends StringTo {

	private String name;						// name of model
	private OGraphic o;							// graphic objec of model

	private Values scale; 						// the scale gap
	private double[] wrap; 						// the wrap values
	private Values resize; 						// the resize value
	private Values morph; 						// the morph value

	private Labels lConstraints;				// the constraints (labels)
	private Vector<Constraint> cConstraints;	// the constraints (Contraint)

	private Values weight;						// weigth of symbol apparition defined by user
	private double fortune;						// fortune of symbol according to the geometric constraint
	private double probability;					// computed according to the weight and fortune.



	/** Default constructor. */
	Model() {
		name = null; o = null;
		scale = new Values(1); wrap = new double[2];
		resize = new Values(1); morph = new Values(1); 
		lConstraints  = new Labels();
		cConstraints = new Vector<Constraint>();
		weight = new Values(1);; fortune = 0.0; probability = 0.0;
		}



	/** To load the url tag. */
	void loadUrl(String dir,String url) {
		if((dir!=null)&(url!=null))
			loadUrl(dir+"/"+new File(url).getName());
		else
			loadUrl(url);
		}
	void loadUrl(String url) {
		if(url != null) {
			file = new File(url);
			if(file.exists()) {
				loadName(file);
				loadGraphic(file);
				}
			}
		}
	private File file;



	/** To get the model name. */
	public String getName() {
		if(name != null)
			return new String(name);
		return null;
		}
	private void loadName(File file) {
		name = file.getName();
		if(name.length() > 4)
			name = name.substring(0,name.length()-4);
		}



	/** To get the graphics object model. */
	public OGraphic getGraphic() {
		if(o != null)
			return o.copy();
		return null;
		}
	private void loadGraphic(File file) {
		if(file.getName().toLowerCase().endsWith(".svg"))
			o = Read.svgRead(file.getPath(),false);
		}



	/** To get the next scale attribute. */
	public double getScale()
		{ return scale.get(); }
	void setScale(String v)
		{ scale.set(v); }



	/** To get the wrap attribute. */
	public double[] getWrap()
		{  wc = new double[2]; wc[0] = wrap[0]; wc[1] = wrap[1]; return wc; }
	private double[] wc;
	void setWrap(String v) {
		if(v != null) {
			String[] tab = v.split(";");
			if(tab.length==2) {
				wrap[0] = getDouble(tab[0]);
				wrap[1] = getDouble(tab[1]);
				}
			}
		}



	/** To set the resize attribute. */
	public double getResize()
		{ return resize.get(); }
	void setResize(String v)
		{ resize.set(v); }

	
	
	
	/** To get the morph attribute. */
	public double getMorph()
		{ return morph.get(); }
	void setMorph(String v)
		{ morph.set(v); }

	
	
	/** To get the weight attribute. */
	public double getWeight()
		{ return weight.get();}
	void setWeight(String v)
		{ weight.set(v); }



	/** To get the fortune attribute. */
	public double getFortune()
		{ return fortune;}
	void setFortune(double fortune) {
		if(fortune>=0)
			this.fortune = fortune;
		}
	void setFortune() {
		fortune=0; Constraint c;
		for(int i=0;i<cConstraints.size();i++) {
			c = cConstraints.get(i);
			fortune+=1/((double)c.modelNumber());
			}
		}



	/** To get the probability attribute. */
	public double getProbability()
		{ return probability; }
	void setProbability(double probability) {
		if((probability>=0)&(probability<=1))
			this.probability = probability;
		}



	/** To get the next constraint attribute. */
	public Constraint nextConstraint() {
		if(cConstraints.size()>0)
			return cConstraints.get(r.nextInt(cConstraints.size()));
		return null;
		}
	private Random r = new Random();
	/** To set the constraints. */
	void setConstraints(String v)
		{ lConstraints.set(v); }
	/** Test if the symbol is constrained by the given constraint name. */
	boolean isConstrained(String v) {
		if(v != null)
			return lConstraints.contains(v);
		return false;
		}
	/** To add a link to a constraint */
	void addConstraint(Constraint c) {
		if(c != null)
			cConstraints.add(c);
		}
	/** To get the link number. */
	int constraintNumber()
		{return cConstraints.size();}
	/** To get a constraint. */
	Constraint getConstraint(int p) {
		if((p >= 0)&(p < cConstraints.size()))
			return cConstraints.get(p);
		return null;
		}
	/** To get a string representation of constraints. */
	public String getConstraints() {
		StringBuffer sb = new StringBuffer();
		for(int i=0;i<cConstraints.size();i++) {
			sb.append(cConstraints.get(i).getName());
			if(i!=cConstraints.size()-1)
				sb.append(";");
			}
		return sb.toString();
		}



	/** Print method. */
	public void print() {
		if((name!=null)&(o!=null)) {
			System.out.println(	"<model"
								+" name=\""+name+"\""
								+" scale=\""+scale.toString()+"\""
								+" wrap=\""+wrap[0]+";"+wrap[1]+"\""
								+" resize=\""+resize.toString()+"\""
								+" morph=\""+morph.toString()+"\""
								+" weight=\""+weight.toString()+"\""
								+" fortune=\""+fortune+"\""
								+" probability=\""+probability+"\""
								+" constraints=\""+getConstraints()+"\""
								+">\n");
			System.out.println("</model>\n");
			}
		}

	}

