package gggt.eg.set;

import java.util.*;

/**
* The Setting class,
* object corresponding to the setting tag
* into the setting file.
*/
public class Setting extends StringTo {

	private String path;					// path of the setting file
	private int size;						// size of database to generate
	private Algorithm algorithm;			// algorithm object
	private Map map;						// map object
	private Noise noise;					// noise object
	private Vector<Model> models;			// model array
	private Vector<Constraint> constraints;	// constraint array

	private Vector<Constraint> rigids; 		// the rigids stack
	private int rcounter;					// the rigids counter

	/** Default constructor. */
	Setting() {
		path = "";
		size = 1;
		algorithm = new Algorithm();
		map = new Map();
		noise = new Noise();
		models = new Vector<Model>();
		constraints = new Vector<Constraint>();

		rigids = new Vector<Constraint>();
		rcounter = 0;
		}

	/** To get the setting file path. */
	public String getPath()
		{return path;}
	void setPath(String path)
		{ if(path!=null) this.path = new String(path); }

	/** To get the size of database to generate. */
	public int getSize()
		{return size;}
	void setSize(String v) {
		if(v != null) {
			int n = getInteger(v);
			if(n>1)
				size = n;
			}
		}

	/** To get the algorithm object. */
	public Algorithm getAlgorithm() {return algorithm;}

	/** To get the map object. */
	public Map getMap() {return map;}

	/** To get the noise object. */
	public Noise getNoise() {return noise;}

	/** To add model/constraint. */
	void addModel(Model m) {
		if(m != null)
			models.add(m);
		}
	void addConstraint(Constraint c) {
		if(c != null)
			constraints.add(c);
		}

	/**
	* To sum the symbols' sizes.
	*/
	void sumSizes() {
		int size = 0;
		for(int i=0;i<constraints.size();i++)
			size += constraints.get(i).getSize();
		map.setSize(size);
		}

	/**
	* To set the links
	* contraints to models
	* and models to constraints.
	*/
	void setLinks() {
		int i,j; Constraint c; Model m;
		for(i=0;i<models.size();i++) {
			m = models.get(i);
			for(j=0;j<constraints.size();j++) {
				c = constraints.get(j);
				if(m.isConstrained(c.getName()))
					{m.addConstraint(c); c.addModel(m);}
				}
			}
		}

	/** To get the rigid constraints. */
	public void setRigids() {
		for(int i=0;i<constraints.size();i++)
			if(constraints.get(i).getRigid()==true)
				rigids.add(constraints.get(i));
		}

	/**
	* To set the fortune parametter of models.
	*/
	void setFortunes() {
		for(int i=0;i<models.size();i++)
			models.get(i).setFortune();
		}

	/**
	* To compute the probability density according
	* to model weights and fortunes.
	* this method must be use after models loading.
	*/
	void rangeModels() {
		int i; Model gs; double ws=0;
		for(i=0;i<models.size();i++) {
			gs = models.get(i);
			ws += gs.getWeight()*gs.getFortune();
			}
		for(i=0;i<models.size();i++) {
			gs = models.get(i);
			gs.setProbability(gs.getWeight()*gs.getFortune()/ws);
			}
		}




	/**
	* The start method
	* clear the constraints' stack,
	* put to zero the
	*/
	public void start() {
		for(int i=0;i<constraints.size();i++)
			constraints.get(i).down();
		rcounter = 0;
		}

	/** To get a next rigid constraint according to a counter. */
	public Constraint nextRigid() {
		if((rigids.size()>0) & (rcounter <rigids.size()))
			{rcounter++; return rigids.get(rcounter-1);}
		return null;
		}

	public boolean hasRigid() {
		if((rigids.size()>0)&(rcounter<rigids.size()))
			return true;
		return false;
		}

	/** To get a next Model according to a computed probability [0,1]. */
	public Model nextModel() {
		double p = Math.abs(new Random().nextDouble()%1);
		int i; double s = 0;
		for(i=0;i<models.size();i++) {
			s += ((Model)models.get(i)).getProbability();
			if(s >= p)
				return (Model)models.get(i);
			}
		return null;
		}




	/** To print the setting object like XML style. */
	public void print() {
		System.out.println	(
							"<setting"
							+" size=\""+size
							+"\">\n"
							);
		algorithm.print();
		map.print();
		noise.print();
		for(int i=0;i<models.size();i++)
			((Model)models.get(i)).print();
		for(int i=0;i<constraints.size();i++)
			((Constraint)constraints.get(i)).print();
		System.out.println("</setting>\n");
		}

	}
