package gggt.eg.set;

import java.io.*;

import org.xml.sax.*;
import javax.xml.parsers.*;
import org.xml.sax.helpers.*;

/**
* The setting handler.
*/
class SettingHandler extends DefaultHandler {

	private Setting set;

	SettingHandler()
		{ set = null; }

	void locateSetting(String fName) {
		if(fName!=null) {
			f = new File(fName);
			if(f.getParent()!=null)
				sRoot = f.getParent()+"/";
			if(f.isFile()&f.exists())
				sPath = f.getPath();
			}
		}
	private File f;
	private String sRoot = "";
	private String sPath = "";

	void locateGrounds(String fName) {
		if(fName!=null)
			if(new File(fName).isDirectory())
				gDirectory = fName;
		}
	private String gDirectory = null;

	void locateModels(String fName) {
		if(fName!=null)
			if(new File(fName).isDirectory())
				mDirectory = fName;
		}
	private String mDirectory = null;

 	public void startDocument() throws SAXException
 		{ set = new Setting(); set.setPath(sPath); }

	public void startElement(String namespaceURI,String lName,String qName,Attributes attrs) throws SAXException {
		if(qName.toLowerCase().equals("setting"))
			settingRead(attrs);
		if(qName.toLowerCase().equals("algorithm"))
			algorithmRead(attrs);
		if(qName.toLowerCase().equals("map"))
			mapRead(attrs);
		if(qName.toLowerCase().equals("noise"))
			noiseRead(attrs);
		if(qName.toLowerCase().equals("model"))
			modelRead(attrs);
		if(qName.toLowerCase().equals("models"))
			modelsRead(attrs);
		if(qName.toLowerCase().equals("fixed"))
			{ c = new Fixed(); constraintRead(c,attrs); }
		if(qName.toLowerCase().equals("sliding"))
			{ c = new Sliding(); constraintRead(c,attrs); }
		if(qName.toLowerCase().equals("zone"))
			{ c = new Zone(); constraintRead(c,attrs); }
		}
	private Constraint c;

	private void settingRead(Attributes attrs) throws SAXException {
		if ((attrs!=null)&(set!=null))
			for (int i = 0; i < attrs.getLength(); i++)
				{ if(attrs.getQName(i).toLowerCase().equals("size")) {set.setSize(attrs.getValue(i));} }
		}

	private void mapRead(Attributes attrs) throws SAXException {
		Map map = set.getMap();
		if((attrs!= null)&(map!=null))
			for (int i = 0; i < attrs.getLength(); i++) {
				if(attrs.getQName(i).toLowerCase().equals("scale")) {map.setScale(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("width")) {map.setWidth(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("height")) {map.setHeight(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("ground")) {map.setGround(gDirectory,sRoot+attrs.getValue(i));}
				}
		}

	private void algorithmRead(Attributes attrs) throws SAXException {
		Algorithm algorithm = set.getAlgorithm();
		if((attrs != null)&(algorithm!=null))
			for (int i = 0; i < attrs.getLength(); i++) {
				if(attrs.getQName(i).toLowerCase().equals("build")) {algorithm.setBuild(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("fuse")) {algorithm.setFuse(attrs.getValue(i));}
				}
		}

	private void noiseRead(Attributes attrs) throws SAXException {
		Noise noise = set.getNoise(); noise.setApply(true);
		if((attrs != null)&(noise!=null))
			for (int i = 0; i < attrs.getLength(); i++) {
				if(attrs.getQName(i).toLowerCase().equals("variance")) {noise.setVariance(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("resize")) {noise.setResize(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("rotation")) {noise.setRotation(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("thickness")) {noise.setThickness(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("move")) {noise.setMove(attrs.getValue(i));}
				}
		}

	private void modelRead(Attributes attrs) throws SAXException {
		if((attrs!=null)&(set!=null)) {
			Model m = new Model();
			for (int i = 0; i < attrs.getLength(); i++) {
				if(attrs.getQName(i).toLowerCase().equals("url")) {m.loadUrl(mDirectory,sRoot+attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("scale")) {m.setScale(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("wrap")) {m.setWrap(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("resize")) {m.setResize(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("morph")) {m.setMorph(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("weight")) {m.setWeight(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("constraints")) {m.setConstraints(attrs.getValue(i));}
				}
			set.addModel(m);
			}
		}

	private void modelsRead(Attributes attrs) throws SAXException {
		if(attrs!=null) {
			Models m = new Models();
			for (int i = 0; i < attrs.getLength(); i++) {
				if(attrs.getQName(i).toLowerCase().equals("url")) {m.loadUrl(mDirectory,sRoot+attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("scale")) {m.setScale(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("wrap")) {m.setWrap(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("resize")) {m.setResize(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("morph")) {m.setMorph(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("weight")) {m.setWeight(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("constraints")) {m.setConstraints(attrs.getValue(i));}
				}
			m.loadModels(set);
			}
		}

	private void constraintRead(Constraint c, Attributes attrs) throws SAXException {
		if((c != null)&(attrs != null)&(set!=null)) {
			for (int i = 0; i < attrs.getLength(); i++) {
				if(attrs.getQName(i).toLowerCase().equals("name")) {c.setName(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("size")) {c.setSize(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("scale")) {c.setScale(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("wrap")) {c.setWrap(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("resize")) {c.setResize(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("morph")) {c.setMorph(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("rigid")) {c.setRigid(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("overflow")) {c.setOverflow(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("overlap")) {c.setOverlap(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("outside")) {c.setOutside(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("roll")) {c.setRoll(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("polar")) {c.setPolar(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("slope")) {c.setSlope(attrs.getValue(i));}
				if(attrs.getQName(i).toLowerCase().equals("shape")) {c.setShape(attrs.getValue(i));}
				}
			set.addConstraint(c);
			}
		}

 	public void endDocument() throws SAXException
 		{ set.sumSizes(); set.setLinks(); set.setRigids(); set.setFortunes(); set.rangeModels(); }

	Setting getSetting()
		{ return set; }

	}

/**
* The setting parser.
*/
public class SettingParser {

	private SettingHandler handler;
	private SAXParser saxParser;

	/** Default constructor */
	public SettingParser() {
        handler = new SettingHandler();
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try
        	{ saxParser = factory.newSAXParser(); }
	    catch(Throwable t) {
			t.printStackTrace();
			System.exit(0);
			}
    	}

	/**
	* Parse method.
	* @param sName the setting file path
	* @return a setting object corresponding to parsing result.
	*/
	public Setting parse(String sName) {
		if(sName!=null)
			if(new File(sName).isFile()) {
				try
					{ handler.locateSetting(sName); saxParser.parse(sName,handler); }
				catch(Throwable t) {
					System.out.println("IOException:"+sName);
					System.exit(0);
					}
				return handler.getSetting();
				}
		return null;
		}

	/**
	* Parse method.
	* @param sName the setting file path
	* @param gName the ground file path
	* @param mName the model directory path
	* @return a setting object corresponding to parsing result.
	*/
	public Setting parse(String sName,String gName,String mName) {
		if((sName!=null)&(gName!=null)&(mName!=null)) {
			handler.locateGrounds(gName);
			handler.locateModels(mName);
			return parse(sName);
			}
		return null;
		}

	}


