package gggt.eg.set;

import java.util.*;

/**
* This class corresponds to a values attribute
* into the setting file. These values can be:
*	[f]ixed : with a tag equals to "0.5" the value will be of {0.5}
*	[g]ap : with a tag equals to "0.5;1.1;0.1" (eq. min;max;gap)
*	the possible values will be of {0.5,0.6,0.7,0.8,0.9,1.0,1.1}
*/
class Values extends StringTo {

	private double []values = new double[3];	// the values (min;max;gap)

	/** Default constructor. */
	Values()
		{clear();}

	/** To clear the double array. */
	private void clear()
		{values[0] = values[1] = values[2] = 0;}

	/** Constructor with a double value. */
	Values(double v)
		{set(v);}

	/** To set values from a double value. */
	private void set(double v) {
		values[0] = values[1] = v;
		values[2] = 0;
		}

	/** Constructor with a double array. */
	Values(double v0,double v1,double v2)
		{set(v0,v1,v2);}

	/** To set values from a double array. */
	private void set(double v0,double v1,double v2) {
		if(check(v0,v1,v2)) {
			values[0] = v0;
			values[1] = v1;
			values[2] = v2;
			}
		else
			clear();
		}

	/** To check a string array. */
	private boolean check(double v0,double v1,double v2) {
		if	(
			((v1-v0)>0)
			&
			((v2>0)&(v2<=Math.abs(v1-v0)))
			)
			return true;
		return false;
		}

	/** Constructor with tag. */
	Values(Values v) {
		if(v!=null) {
			values[0] = v.min();
			values[1] = v.max();
			values[2] = v.gap();
			}
		else
			clear();
		}

	/** Constructor with string. */
	Values(String v)
		{set(v);}

	/** To set values from a string value. */
	void set(String v) {
		if(v != null) {
			String[] tab = v.split(";");
			if(check(tab) > 0) {
				if(tab.length == 1)
					set(getDouble(tab[0]));
				else if(tab.length == 3)
					set(getDouble(tab[0]),getDouble(tab[1]),getDouble(tab[2]));
				else
					clear();
				}
			else
				clear();
			}
		else
			clear();
		}
	private int check(String[] array) {
		if(array != null) {
			for(int i=0;i<array.length;i++)
				if(array[i] == null)
					return -1;
			return array.length;
			}
		return -1;
		}

	/** To get the min value. */
	double min()
		{return values[0];}
	/** To get the max value. */
	double max()
		{return values[1];}
	/** To get the mean value. */
	double mean()
		{return (values[1]-values[0])/2;}
	/** To get the gap value. */
	double gap()
		{return values[2];}

	/** To get a random value. */
	double get() {
		if(isGap()) {
			double ds = values[1]-values[0];
			ds = ds * (Math.abs(new Random().nextDouble()%1));
			ds = (double)Math.round(ds/values[2])*values[2];
			return values[0] + ds;
			}
		return values[1];
		}
	/** Is it a gap or a fixed value. */
	private boolean isGap()
		{return check(values[0],values[1],values[2]);}

	/**
	* Test if the parametter value
	* belongs the values.
	* v == i*gap and min <= v <= max
	*/
	boolean belongs(double v) {
		if(isGap()) {
			for(double i = values[0]; i <= values[1]; i+= values[2])
				if(i==v)
					return true;
			}
		else if(v == values[1])
			return true;
		return false;
		}

	/**
	* Test if the parametter value
	* joins the values.
	* min <= v <= max.
	*/
	boolean joins(double v) {
		if((v>=values[0])&(v<=values[1]))
			return true;
		return false;
		}

	/** To get a string representation. */
	public String toString()
		{return String.valueOf(min())+";"+String.valueOf(max())+";"+String.valueOf(gap());}
	}